package stftr.protocols.Game;

import appia.*;
import appia.events.*;
import appia.events.channel.ChannelInit;
import appia.protocols.udpsimple.RegisterSocketEvent;
import stftr.protocols.events.*;
import stftr.server.GameServer;

/**
* This class is responsible for managing the interaction with the game.
*/
public class GameSession extends Session {
    Channel channel;
    GameServer gameServer;
    int port;
    
    public GameSession(GameLayer l, GameServer gameServer, int port) {
        super(l);
        this.gameServer=gameServer;
        this.port=port;
        channel=null;
    }
    
    /**
    * Handles the events that appia sends.
    */
    public void handle(Event e) {
        if (e instanceof ChannelInit) {
            handle((ChannelInit) e);
        } else if (e instanceof ActionEvent) {
            handle((ActionEvent) e);
        } else {
            System.err.println("[GameSession] Received an unexpected event...\n\t"+e);
            System.exit(1);
        }
        
        try {
            e.go();
        } catch (Exception ex) {
            System.err.println("[GameSession] Error in e.go().");
            ex.printStackTrace();
            System.exit(1);
        }
    }
    
    /**
    * Initializes the chanel.
    */
    void handle(ChannelInit e) {
        channel=e.getChannel();
        
        try {
            e.go();
            RegisterSocketEvent rse=new RegisterSocketEvent(e.getChannel(), new Direction(Direction.DOWN), this, port);
            rse.init();
            rse.go();
        } catch (Exception ex) {
            System.err.println("[GameSession] Error initializing the channel.");
            ex.printStackTrace();
            System.exit(1);
        }
    }
    
    /**
    * Checks the desired action and executes it.
    */
    void handle(ActionEvent e) {
        if (gameServer==null)
            return; // This never happes. Received an event prior to inicialize the board...
        
        if (e instanceof PlayerActionEvent) {
            handle((PlayerActionEvent) e);
        } else if (e instanceof AddPlayerEvent) {
            handle((AddPlayerEvent) e);
        } else { // instanceof RemovePlayerEvent
            handle((RemovePlayerEvent) e);
        }
    }

    /**
    * Executes a player action.
    */
    void handle(PlayerActionEvent e) {
        if (e.getAction()==PlayerActionEvent.SHOOT) {
            Integer shooted=gameServer.shoot(e.getPlayerID());
            if (shooted.intValue()!=-1) {
                try {
                    KilledPlayerEvent killedEvent=new KilledPlayerEvent(e.getChannel(), new Direction(Direction.DOWN), this);
                    killedEvent.dest=shooted;
                    killedEvent.init();
                    killedEvent.go();
                } catch (Exception ex) {
                    System.err.println("[GameSession] Could not inform the player killed.");
                    ex.printStackTrace();
                    System.exit(1);
                }
            }
        } else
            gameServer.movePlayer(e.getPlayerID(), e.getAction());
        
        try {
            BoardEvent boardEvent=new BoardEvent(gameServer.board, e.getPlayerID(), channel, new Direction(Direction.DOWN), this);
            boardEvent.init();
            boardEvent.go();
        } catch (Exception ex) {
            System.err.println("[GameSession] Could not propagate the action taken.");
            ex.printStackTrace();
            System.exit(1);
        }
    }

    /**
    * Adds a player to the server.
    */
    void handle(AddPlayerEvent e) {
        try {
            Integer playerID=gameServer.addPlayer();
            AddedPlayerEvent addedPlayerEvent=new AddedPlayerEvent(playerID, channel, new Direction(Direction.DOWN), this);
            addedPlayerEvent.dest=e.source;
            addedPlayerEvent.init();
            addedPlayerEvent.go();

            System.out.println("Vou enviar uma board com playerID ["+playerID+"]");
            BoardEvent boardEvent=new BoardEvent(gameServer.board, playerID, channel, new Direction(Direction.DOWN), this);
            boardEvent.init();
            boardEvent.go();
        } catch (Exception ex) {
            System.err.println("[GameSession] Could not propagate the added player ID.");
            ex.printStackTrace();
            System.exit(1);
        }
    }

    /**
    * Removes a player from the server.
    */
    void handle(RemovePlayerEvent e) {
        gameServer.removePlayer(e.getPlayerID());
        try {
            BoardEvent boardEvent=new BoardEvent(gameServer.board, e.getPlayerID(), channel, new Direction(Direction.DOWN), this);
            boardEvent.init();
            boardEvent.go();
        } catch (Exception ex) {
            System.err.println("[GameSession] Could not propagate the board.");
            ex.printStackTrace();
            System.exit(1);
        }
    }
}