package stftr.protocols.Player;

import appia.*;
import appia.events.*;
import appia.events.channel.ChannelInit;
import appia.protocols.udpsimple.RegisterSocketEvent;
import appia.protocols.udpsimple.InetWithPort;
import appia.protocols.tcpcomplete.*;
import stftr.protocols.events.*;
import stftr.player.GameBoardComponent;
import stftr.Board;

public class PlayerSession extends Session {
    /** Used to move player UP in doAction() */
    public final static int UP=PlayerActionEvent.UP;
    /** Used to move player DOWN in doAction() */
    public final static int DOWN=PlayerActionEvent.DOWN;
    /** Used to move player LEFT in doAction() */
    public final static int LEFT=PlayerActionEvent.LEFT;
    /** Used to move player RIGHT in doAction() */
    public final static int RIGHT=PlayerActionEvent.RIGHT;
    /** Used to make player SHOOT in doAction() */
    public final static int SHOOT=PlayerActionEvent.SHOOT;

    GameBoardComponent boardComponent;
    Channel channel;
    InetWithPort serverInetWithPort;

    PlayerSession(PlayerLayer l, GameBoardComponent boardComponent) {
        super(l);
        this.boardComponent=boardComponent;
        channel=null;
        serverInetWithPort=null;
    }

    public void handle(Event e) {
        if (e instanceof ChannelInit) {
            channel=e.getChannel();
        } else if (e instanceof AddedPlayerEvent) {
            boardComponent.playerID=((AddedPlayerEvent) e).getPlayerID();
        } else if (e instanceof KilledPlayerEvent) {
            System.out.println("Foste morto -> GAME OVER");
            System.exit(0);
        } else if (e instanceof BoardEvent) {
            boardComponent.setBoard(((BoardEvent) e).getBoard());
            if (((BoardEvent) e).getPlayerID().equals(boardComponent.playerID))
                boardComponent.startListen();
        } else if (e instanceof TcpUndeliveredEvent) {
                System.err.println("[PlayerSession] TCP undelivered event. [Probably server down]"+e);
                System.exit(1);
        }

        try {
            e.go();
        } catch (Exception ex) {
                System.err.println("[PlayerSession] Error in e.go().");
                ex.printStackTrace();
                System.exit(1);
        }
    }

    public void connectToServer(String host, int port) {
        try {
            RegisterSocketEvent rse=new RegisterSocketEvent(channel, new Direction(Direction.DOWN), this);
            rse.init();
            rse.go();

            AddPlayerEvent addPlayerEvent=new AddPlayerEvent(channel, new Direction(Direction.DOWN), this);
            serverInetWithPort=new InetWithPort();
            serverInetWithPort.host=java.net.InetAddress.getByName(host);
            serverInetWithPort.port=port;
            addPlayerEvent.dest=serverInetWithPort;
            addPlayerEvent.init();
            addPlayerEvent.go();
        } catch (java.net.UnknownHostException ex) {
                System.err.println("[PlayerSession] Unknow host ["+host+"] for the game server.");
                ex.printStackTrace();
                System.exit(1);
        } catch (Exception ex) {
                System.err.println("[PlayerSession] Error while trying to connect to the server.");
                ex.printStackTrace();
                System.exit(1);
        }
    }

    public void doAction(int action) {
        boardComponent.stopListen();

        try {
            PlayerActionEvent playerActionEvent=new PlayerActionEvent(boardComponent.playerID, action, channel, new Direction(Direction.DOWN), this);
            playerActionEvent.dest=serverInetWithPort;
            playerActionEvent.init();
            playerActionEvent.go();
        } catch (Exception ex) {
                System.err.println("[PlayerSession] Error while trying to execute the selected action ["+action+"].");
                ex.printStackTrace();
                System.exit(1);
        }
    }
}