package stftr.server;

import java.util.*;
import stftr.Board;

/**
* Keeps the position...
*/
class Info {
    int x,y;

    Info(int x, int y) {
        this.x=x;
        this.y=y;
    }
}

/**
* This class is responsible for managing a game.
*/
public class GameServer {
    /** next playerID */
    private int playerID;
    /** random to generate positions on the board for a new player */
    private Random random;
    /** key: Integer (playerID) data: Info */
    private Hashtable player;

    /** The game board (should be read-only outside the class) */
    public Board board;

    /**
    * Instantiates a new GameServer.
    *
    * @param board A game board where the game will take place. The board must
    * be an array with dimensions [Board.HEIGHT][Board.WIDTH]. The smaller the height,
    * higher on the screen. The smaller the width, "lefter" on the screen.
    */
    public GameServer(Board board) {
        random=new Random();
        this.board=board;
        player=new Hashtable();
        playerID=0;
    }

    /**
    * Adds a player to the game board in a random position.
    *
    * @return The identifier of the player.
    */
    public Integer addPlayer() {
        int x,y;

        playerID++;
        playerID&=0x0FFFFFFF; // FIXME: comentar em ingles -> d a volta mais cedo...

        do {
            x=random.nextInt(Board.WIDTH);
            y=random.nextInt(Board.HEIGHT);
        }
        while (board.getType(x,y)!=Board.EMPTY);

        Info info=new Info(x, y);
        Integer nPlayer=new Integer(playerID);
        player.put(nPlayer, info);

        board.putBoard(x, y, Board.PLAYER, Board.RIGHT, playerID);

        return nPlayer;
    }

    /**
    * Removes the player.
    */
    public void removePlayer(Integer playerID) {
        if (!player.containsKey(playerID))
            return;

        Info info=(Info) player.remove(playerID);
        board.putBoard(info.x, info.y, Board.EMPTY, 0, 0);
    }

    /**
    * Moves a player in a given direction. If the move isn't possibe (end of
    * the board or way is blocked) the players stays on the same position but
    * is direction is changed in accordance.
    *
    * @param nPlayer The playerID of the player that wishes to move.
    * @param direction The direction the player wishes to go.
    */
    public void movePlayer(Integer playerID, int direction) {
        if (!player.containsKey(playerID))
            return;

        Info info=(Info) player.get(playerID);
        switch (direction) {
            case Board.UP:
                if (info.y==0 || board.getType(info.x, info.y-1)!=Board.EMPTY)
                    board.putBoard(info.x, info.y, Board.PLAYER, Board.UP, playerID.intValue());
                else {
                    board.putBoard(info.x, info.y, Board.EMPTY, 0, 0);
                    board.putBoard(info.x, --info.y, Board.PLAYER, Board.UP, playerID.intValue());
                }
                break;

            case Board.DOWN:
                if (info.y==Board.HEIGHT-1 || board.getType(info.x, info.y+1)!=Board.EMPTY)
                    board.putBoard(info.x, info.y, Board.PLAYER, Board.DOWN, playerID.intValue());
                else {
                    board.putBoard(info.x, info.y, Board.EMPTY, 0,  0);
                    board.putBoard(info.x, ++info.y, Board.PLAYER, Board.DOWN, playerID.intValue());
                }
                break;

            case Board.LEFT:
                if (info.x==0 || board.getType(info.x-1, info.y)!=Board.EMPTY)
                    board.putBoard(info.x, info.y, Board.PLAYER, Board.LEFT, playerID.intValue());
                else {
                    board.putBoard(info.x, info.y, Board.EMPTY, 0, 0);
                    board.putBoard(--info.x, info.y, Board.PLAYER, Board.LEFT, playerID.intValue());
                }
                break;

            case Board.RIGHT:
                if (info.x==Board.WIDTH-1 || board.getType(info.x+1, info.y)!=Board.EMPTY)
                    board.putBoard(info.x, info.y, Board.PLAYER, Board.RIGHT, playerID.intValue());
                else {
                    board.putBoard(info.x, info.y, Board.EMPTY, 0, 0);
                    board.putBoard(++info.x, info.y, Board.PLAYER, Board.RIGHT, playerID.intValue());
                }
                break;
        }
    }

    /**
    * Shoots...
    *
    * @param playerID The playerID of the player who wishes to shoot. -1 if none.
    */
    public Integer shoot(Integer playerID) {
        Integer kill=new Integer(-1);

        if (player.containsKey(playerID)) {
            Info info=(Info) player.get(playerID);
            switch (board.getDirection(info.x, info.y)) {
                case Board.UP:
                    for (int y=info.y-1; y>=0; y--) {
                        if (board.getType(info.x, y)==Board.PLAYER) {
                            kill=new Integer(board.getID(info.x, y));
                            board.putBoard(info.x, y, Board.EMPTY, 0, 0);
                            player.remove(kill);
                            break;
                        }
                    }
                    break;

                case Board.DOWN:
                    for (int y=info.y+1; y<Board.HEIGHT; y++) {
                        if (board.getType(info.x, y)==Board.PLAYER) {
                            kill=new Integer(board.getID(info.x, y));
                            board.putBoard(info.x, y, Board.EMPTY, 0, 0);
                            player.remove(kill);
                            break;
                        }
                    }
                    break;

                case Board.LEFT:
                    for (int x=info.x-1; x>=0; x--) {
                        if (board.getType(x, info.y)==Board.PLAYER) {
                            kill=new Integer(board.getID(x, info.y));
                            board.putBoard(x, info.y, Board.EMPTY, 0, 0);
                            player.remove(kill);
                            break;
                        }
                    }
                    break;

                case Board.RIGHT:
                    for (int x=info.x+1; x<Board.WIDTH; x++) {
                        if (board.getType(x, info.y)==Board.PLAYER) {
                            kill=new Integer(board.getID(x, info.y));
                            board.putBoard(x, info.y, Board.EMPTY, 0, 0);
                            player.remove(kill);
                            break;
                        }
                    }
                    break;
            }
        }

        return kill;
    }
}